@@ -1,4 +1,4 @@
-//  StaticRcRef
+//! `StaticRcRef` is a compile-time referenced counted access to a mutable reference.
 
 use core::{
     any,
@@ -41,23 +41,46 @@ pub struct StaticRcRef<'a, T: ?Sized, const NUM: usize, const DEN: usize> {
     _marker: PhantomData<&'a mut T>,
 }
 
-impl<'a, T, const N: usize> StaticRcRef<'a, T, N, N> {
+impl<'a, T: ?Sized, const N: usize> StaticRcRef<'a, T, N, N> {
     /// Constructs a new `StaticRcRef<'a, T, N, N>`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 1, 1>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// assert_eq!(42, *rc);
+    /// ```
     #[inline(always)]
-    pub fn new(value: &'a mut T) -> Self {
+    pub fn new(value: &'a mut T) -> Self
+    where
+        AssertLeType!(1, N): Sized,
+    {
+        #[cfg(not(feature = "compile-time-ratio"))]
+        assert!(N > 0);
+
         let pointer = NonNull::from(value);
         Self { pointer, _marker: PhantomData }
     }
 
-    /// Constructs a new `Pin<StaticRcRef<'a, T, N, N>>`.
-    #[inline(always)]
-    pub fn pin(value: &'a mut T) -> pin::Pin<Self> {
-        //  Safety:
-        //  -   The `value` is mutably borrowed, and cannot be moved out of without full ownership.
-        unsafe { pin::Pin::new_unchecked(Self::new(value)) }
-    }
-
     /// Returns the inner value.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 1, 1>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let inner: &mut i32 = Full::into_inner(rc);
+    /// assert_eq!(42, *inner);
+    /// ```
     #[inline(always)]
     pub fn into_inner(this: Self) -> &'a mut T {
         //  Safety:
@@ -65,10 +88,20 @@ impl<'a, T, const N: usize> StaticRcRef<'a, T, N, N> {
         //  -   Original lifetime is restored.
         unsafe { &mut *this.pointer.as_ptr() }
     }
-}
 
-impl<'a, T: ?Sized, const N: usize> StaticRcRef<'a, T, N, N> {
     /// Returns a mutable reference into the given `StaticRcRef`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 1, 1>;
+    ///
+    /// let mut value = 42;
+    /// let mut rc = Full::new(&mut value);
+    /// assert_eq!(42, *Full::get_mut(&mut rc));
+    /// ```
     #[inline(always)]
     pub fn get_mut(this: &mut Self) -> &mut T {
         //  Safety:
@@ -79,16 +112,60 @@ impl<'a, T: ?Sized, const N: usize> StaticRcRef<'a, T, N, N> {
 
 impl<'a, T: ?Sized, const NUM: usize, const DEN: usize> StaticRcRef<'a, T, NUM, DEN> {
     /// Consumes the `StaticRcRef`, returning the wrapped pointer.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 1, 1>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let pointer = Full::into_raw(rc);
+    /// assert_eq!(&mut value as *mut _, pointer.as_ptr());
+    /// ```
     #[inline(always)]
     pub fn into_raw(this: Self) -> NonNull<T> { this.pointer }
 
     /// Provides a raw pointer to the data.
     ///
     /// `StaticRcRef` is not consumed or affected in any way, the pointer is valid as long as the original value is.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 1, 1>;
+    ///
+    /// let mut value = 42;
+    /// let pointer = &mut value as *mut _;
+    ///
+    /// let rc = Full::new(&mut value);
+    /// let other_pointer = Full::as_ptr(&rc);
+    ///
+    /// assert_eq!(pointer, other_pointer.as_ptr());
+    /// ```
     #[inline(always)]
     pub fn as_ptr(this: &Self) -> NonNull<T> { this.pointer }
 
     /// Provides a reference to the data.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 1, 1>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    ///
+    /// let r = Full::get_ref(&rc);
+    ///
+    /// assert_eq!(42, *r);
+    /// ```
     #[inline(always)]
     pub fn get_ref(this: &Self) -> &T {
         //  Safety:
@@ -105,9 +182,31 @@ impl<'a, T: ?Sized, const NUM: usize, const DEN: usize> StaticRcRef<'a, T, NUM,
     /// -   If `N / D` is different from `NUM / DEN`, then specific restrictions apply. The user is responsible for
     ///     ensuring proper management of the ratio of shares, and ultimately that the value is not dropped twice.
     #[inline(always)]
-    pub unsafe fn from_raw(pointer: NonNull<T>) -> Self { Self { pointer, _marker: PhantomData, } }
+    pub unsafe fn from_raw(pointer: NonNull<T>) -> Self
+    where
+        AssertLeType!(1, NUM): Sized,
+    {
+        #[cfg(not(feature = "compile-time-ratio"))]
+        assert!(NUM > 0);
+
+        Self { pointer, _marker: PhantomData, }
+    }
 
     /// Returns true if the two `StaticRcRef` point to the same allocation.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 2, 2>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let (one, two) = Full::split::<1, 1>(rc);
+    ///
+    /// assert!(StaticRcRef::ptr_eq(&one, &two));
+    /// ```
     #[inline(always)]
     pub fn ptr_eq<const N: usize, const D: usize>(this: &Self, other: &StaticRcRef<'a, T, N, D>) -> bool {
         StaticRcRef::as_ptr(this) == StaticRcRef::as_ptr(other)
@@ -118,13 +217,31 @@ impl<'a, T: ?Sized, const NUM: usize, const DEN: usize> StaticRcRef<'a, T, NUM,
     /// #   Panics
     ///
     /// If the compile-time-ratio feature is not used, and the ratio is not preserved; that is `N / D <> NUM / DEN`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 2, 2>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let rc = Full::adjust::<1, 1>(rc);
+    ///
+    /// assert_eq!(42, *rc);
+    /// ```
     #[inline(always)]
     pub fn adjust<const N: usize, const D: usize>(this: Self) -> StaticRcRef<'a, T, N, D>
     where
+        AssertLeType!(1, N): Sized,
         AssertEqType!(N * DEN, NUM * D): Sized,
     {
         #[cfg(not(feature = "compile-time-ratio"))]
-        assert_eq!(NUM * D, N * DEN, "{} / {} != {} / {}", NUM, DEN, N, D);
+        {
+            assert!(N > 0);
+            assert_eq!(NUM * D, N * DEN, "{} / {} != {} / {}", NUM, DEN, N, D);
+        }
 
         StaticRcRef { pointer: this.pointer, _marker: PhantomData }
     }
@@ -134,13 +251,34 @@ impl<'a, T: ?Sized, const NUM: usize, const DEN: usize> StaticRcRef<'a, T, NUM,
     /// #   Panics
     ///
     /// If the compile-time-ratio feature is not used, and the ratio is not preserved; that is `A + B <> NUM`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 2, 2>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let (one, two) = Full::split::<1, 1>(rc);
+    ///
+    /// assert_eq!(42, *one);
+    /// assert_eq!(42, *two);
+    /// ```
     #[inline(always)]
     pub fn split<const A: usize, const B: usize>(this: Self) -> (StaticRcRef<'a, T, A, DEN>, StaticRcRef<'a, T, B, DEN>)
     where
+        AssertLeType!(1, A): Sized,
+        AssertLeType!(1, B): Sized,
         AssertEqType!(A + B, NUM): Sized,
     {
         #[cfg(not(feature = "compile-time-ratio"))]
-        assert_eq!(NUM, A + B, "{} != {} + {}", NUM, A, B);
+        {
+            assert!(A > 0);
+            assert!(B > 0);
+            assert_eq!(NUM, A + B, "{} != {} + {}", NUM, A, B);
+        }
 
         let pointer = this.pointer;
         let _marker = PhantomData;
@@ -148,24 +286,40 @@ impl<'a, T: ?Sized, const NUM: usize, const DEN: usize> StaticRcRef<'a, T, NUM,
         (StaticRcRef { pointer, _marker, }, StaticRcRef { pointer, _marker, })
     }
 
-    /// Splits the current instance into two instances with the specified NUMerators.
+    /// Splits the current instance into DIM instances with the specified Numerators.
     ///
     /// #   Panics
     ///
-    /// If the compile-time-ratio feature is not used, and the ratio is not preserved; that is `N * DIM / D <> NUM / DEN`.
+    /// If the compile-time-ratio feature is not used, and the ratio is not preserved; that is `N * DIM <> NUM`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 2, 2>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let array = Full::split_array::<1, 2>(rc);
+    ///
+    /// assert_eq!(42, *array[0]);
+    /// assert_eq!(42, *array[1]);
+    /// ```
     #[inline(always)]
-    pub fn split_array<const N: usize, const D: usize, const DIM: usize>(this: Self) -> [StaticRcRef<'a, T, N, D>; DIM]
+    pub fn split_array<const N: usize, const DIM: usize>(this: Self) -> [StaticRcRef<'a, T, N, DEN>; DIM]
     where
         T: 'a,
-        AssertEqType!(N * DIM * DEN, NUM * D): Sized,
-        AssertLeType!(mem::size_of::<[StaticRcRef<'a, T, N, D>; DIM]>(), usize::MAX / 2 + 1): Sized,
+        AssertEqType!(N * DIM, NUM): Sized,
+        AssertLeType!(mem::size_of::<[StaticRcRef<'a, T, N, DEN>; DIM]>(), usize::MAX / 2 + 1): Sized,
     {
         #[cfg(not(feature = "compile-time-ratio"))]
-        assert_eq!(NUM * D, N * DIM * DEN, "{} * {} != {} * {} * {}", NUM, D, N, DIM, DEN);
+        {
+            assert_eq!(NUM, N * DIM, "{} != {} * {}", NUM, N, DIM);
 
-        #[cfg(not(feature = "compile-time-ratio"))]
-        assert!(mem::size_of::<[StaticRcRef<T, N, D>; DIM]>() <= (isize::MAX as usize),
-            "Size of result ({}) exceeeds isize::MAX", mem::size_of::<[StaticRcRef<T, N, D>; DIM]>());
+            assert!(mem::size_of::<[StaticRcRef<T, N, DEN>; DIM]>() <= (isize::MAX as usize),
+                "Size of result ({}) exceeeds isize::MAX", mem::size_of::<[StaticRcRef<T, N, DEN>; DIM]>());
+        }
 
         let pointer = this.pointer;
         let _marker = PhantomData;
@@ -177,7 +331,7 @@ impl<'a, T: ?Sized, const NUM: usize, const DEN: usize> StaticRcRef<'a, T, NUM,
             //  -   `destination` within bounds of allocated array (< DIM).
             //  -   Offset doesn't overflow `isize`, as per array-size assertion.
             //  -   Offset doesn't wrap around, as per array-size assertion.
-            let destination = unsafe { (array.as_mut_ptr() as *mut StaticRcRef<T, N, D>).add(i) };
+            let destination = unsafe { (array.as_mut_ptr() as *mut StaticRcRef<T, N, DEN>).add(i) };
 
             //  Safety:
             //  -   `destination` is valid for writes.
@@ -197,10 +351,26 @@ impl<'a, T: ?Sized, const NUM: usize, const DEN: usize> StaticRcRef<'a, T, NUM,
     /// If the two instances do no point to the same allocation, as determined by `StaticRcRef::ptr_eq`.
     ///
     /// If the compile-time-ratio feature is not used and the ratio is not preserved; that is `A + B <> NUM`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 2, 2>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let (one, two) = Full::split::<1, 1>(rc);
+    /// let rc = Full::join(one, two);
+    ///
+    /// assert_eq!(42, *rc);
+    /// ```
     #[inline(always)]
     pub fn join<const A: usize, const B: usize>(left: StaticRcRef<'a, T, A, DEN>, right: StaticRcRef<'a, T, B, DEN>) -> Self
-    where
-        AssertEqType!(NUM, A + B): Sized,
+    //  FIXME: re-enable when https://github.com/rust-lang/rust/issues/77708 fixed
+    //  where
+    //      AssertEqType!(NUM, A + B): Sized,
     {
         assert!(StaticRcRef::ptr_eq(&left, &right), "{:?} != {:?}", left.pointer.as_ptr(), right.pointer.as_ptr());
 
@@ -220,47 +390,123 @@ impl<'a, T: ?Sized, const NUM: usize, const DEN: usize> StaticRcRef<'a, T, NUM,
     /// #   Panics
     ///
     /// If the compile-time-ratio feature is not used and the ratio is not preserved; that is `A + B <> NUM`.
+    ///
+    /// In debug, if the two instances do no point to the same allocation, as determined by `StaticRcRef::ptr_eq`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 2, 2>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let (one, two) = Full::split::<1, 1>(rc);
+    /// let rc = unsafe { Full::join_unchecked(one, two) };
+    ///
+    /// assert_eq!(42, *rc);
+    /// ```
     #[inline(always)]
     pub unsafe fn join_unchecked<const A: usize, const B: usize>(
         left: StaticRcRef<'a, T, A, DEN>,
         _right: StaticRcRef<'a, T, B, DEN>,
     ) -> Self
-    where
-        AssertEqType!(NUM, A + B): Sized,
+    //  FIXME: re-enable when https://github.com/rust-lang/rust/issues/77708 fixed
+    //  where
+    //      AssertEqType!(NUM, A + B): Sized,
     {
-        #[cfg(not(feature = "compile-time-ratio"))]
+        //  FIXME: re-enable when https://github.com/rust-lang/rust/issues/77708 fixed
+        //  #[cfg(not(feature = "compile-time-ratio"))]
         assert_eq!(NUM, A + B, "{} != {} + {}", NUM, A, B);
 
+        debug_assert!(StaticRcRef::ptr_eq(&left, &_right), "{:?} != {:?}", left.pointer.as_ptr(), _right.pointer.as_ptr());
+
         Self { pointer: left.pointer, _marker: PhantomData, }
     }
 
-    /// Lifts `this` into the slot provided by `fun`; returns the previous value of the slot, if any.
+    /// Joins DIM instances into a single instance.
     ///
-    /// This function is useful, for example, to "tie the knot" when appending 2 linked-lists: it is easy to splice the
-    /// the head of the back linked-list at the back of the front linked-list, but then one has lost the head pointer
-    /// and can no longer splice the tail of the front linked-list to it.
-    pub fn lift<'b, F>(this: Self, fun: F) -> Option<Self>
-    where
-        'a: 'b,
-        T: 'b,
-        F: FnOnce(&'b Self) -> &'b mut Option<Self>,
+    /// #   Panics
+    ///
+    /// If all instances do not point to the same allocation, as determined by `StaticRcRef::ptr_eq`.
+    ///
+    /// If the compile-time-ratio feature is not used and the ratio is not preserved; that is `N * DIM <> NUM`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 2, 2>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let array = Full::split_array::<1, 2>(rc);
+    /// let rc = Full::join_array(array);
+    ///
+    /// assert_eq!(42, *rc);
+    /// ```
+    #[inline(always)]
+    pub fn join_array<const N: usize, const DIM: usize>(array: [StaticRcRef<'a, T, N, DEN>; DIM]) -> Self
+    //  FIXME: re-enable when https://github.com/rust-lang/rust/issues/77708 fixed
+    //  where
+    //      AssertLeType!(1, NUM): Sized,
+    //      AssertEqType!(N * DIM, NUM): Sized,
     {
-        let slot = {
-            //  Safety:
-            //  -   Even though the lifetime of `this_ref` and `this` was "unlinked" by going through a pointer,
-            //      `this_ref` will notbe used after moving `this`, and therefore will remain valid throughout its used.
-            let this_ref = unsafe { &*(&this as *const _) };
+        let first = &array[0];
+        for successive in &array[1..] {
+            assert!(StaticRcRef::ptr_eq(&first, &successive),
+                "{:?} != {:?}", first.pointer.as_ptr(), successive.pointer.as_ptr());
+        }
 
-            fun(this_ref) as *mut Option<Self>
-        };
+        unsafe { Self::join_array_unchecked(array) }
+    }
 
-        //  Safety:
-        //  -   Even though the lifetime of `slot` and `this` was "unlinked" by going through a pointer, `slot` is
-        //      still validfor the duration of this function because `this` is not destroyed, and therefore its
-        //      pointee is still alive.
-        let slot = unsafe { &mut *slot };
+    /// Joins DIM instances into a single instance.
+    ///
+    /// #   Panics
+    ///
+    /// If the compile-time-ratio feature is not used and the ratio is not preserved; that is `N * DIM <> NUM`.
+    ///
+    /// In debug, if all instances do not point to the same allocation, as determined by `StaticRcRef::ptr_eq`.
+    ///
+    /// #   Example
+    ///
+    /// ```rust
+    /// use static_rc::StaticRcRef;
+    ///
+    /// type Full<'a> = StaticRcRef<'a, i32, 2, 2>;
+    ///
+    /// let mut value = 42;
+    /// let rc = Full::new(&mut value);
+    /// let array = Full::split_array::<1, 2>(rc);
+    /// let rc = unsafe { Full::join_array_unchecked(array) };
+    ///
+    /// assert_eq!(42, *rc);
+    /// ```
+    #[inline(always)]
+    pub unsafe fn join_array_unchecked<const N: usize, const DIM: usize>(array: [StaticRcRef<'a, T, N, DEN>; DIM])
+        -> Self
+    //  FIXME: re-enable when https://github.com/rust-lang/rust/issues/77708 fixed
+    //  where
+    //      AssertLeType!(1, NUM): Sized,
+    //      AssertEqType!(N * DIM, NUM): Sized,
+    {
+        //  FIXME: re-enable when https://github.com/rust-lang/rust/issues/77708 fixed
+        //  #[cfg(not(feature = "compile-time-ratio"))]
+        {
+            assert!(NUM > 0);
+            assert_eq!(NUM, N * DIM, "{} != {} * {}", NUM, N, DIM);
+        }
 
-        slot.replace(this)
+        let _first = &array[0];
+        for _successive in &array[1..] {
+            debug_assert!(StaticRcRef::ptr_eq(&_first, &_successive),
+                "{:?} != {:?}", _first.pointer.as_ptr(), _successive.pointer.as_ptr());
+        }
+
+        Self { pointer: array[0].pointer, _marker: PhantomData, }
     }
 }
 
@@ -496,13 +742,13 @@ unsafe impl<'a, T: ?Sized + marker::Send, const NUM: usize, const DEN: usize> ma
 
 unsafe impl<'a, T: ?Sized + marker::Sync, const NUM: usize, const DEN: usize> marker::Sync for StaticRcRef<'a, T, NUM, DEN> {}
 
-#[cfg(test)]
-mod tests {
+#[doc(hidden)]
+pub mod compile_tests {
 
 /// ```compile_fail,E0597
 /// let a = String::from("foo");
 /// let mut a_ref = &a;
-/// let mut rc = static_rc::StaticRcRef::<_,1,1>::new(&mut a_ref);
+/// let mut rc = static_rc::StaticRcRef::<'_, _,1,1>::new(&mut a_ref);
 /// {
 ///     let b = String::from("bar");
 ///     *rc = &b; // a_ref now points to b
@@ -510,8 +756,281 @@ mod tests {
 /// // b is now dropped
 /// assert_ne!(a_ref, "bar");  // This should fail to compile.
 /// ```
-fn test_use_after_free() {
-    #![allow(dead_code)]
+pub fn rcref_prevent_use_after_free() {}
+
+} // mod compile_tests
+
+#[doc(hidden)]
+#[cfg(feature = "compile-time-ratio")]
+pub mod compile_ratio_tests {
+
+/// ```compile_fail,E0080
+/// type Zero<'a> = static_rc::StaticRcRef<'a, i32, 0, 0>;
+///
+/// let mut value = 42;
+///
+/// Zero::new(&mut value);
+/// ```
+pub fn rcref_new_zero() {}
+
+/// ```compile_fail,E0080
+/// type Zero<'a> = static_rc::StaticRcRef<'a, i32, 0, 0>;
+///
+/// let pointer = core::ptr::NonNull::dangling();
+///
+/// unsafe { Zero::from_raw(pointer) };
+/// ```
+pub fn rcref_from_raw_zero() {}
+
+/// ```compile_fail,E0080
+/// type One<'a> = static_rc::StaticRcRef<'a, i32, 1, 1>;
+///
+/// let mut value = 42;
+/// let rc = One::new(&mut value);
+///
+/// One::adjust::<0, 0>(rc);
+/// ```
+pub fn rcref_adjust_zero() {}
+
+/// ```compile_fail,E0080
+/// type One<'a> = static_rc::StaticRcRef<'a, i32, 1, 1>;
+///
+/// let mut value = 42;
+/// let rc = One::new(&mut value);
+///
+/// One::adjust::<2, 3>(rc);
+/// ```
+pub fn rcref_adjust_ratio() {}
+
+/// ```compile_fail,E0080
+/// type Two<'a> = static_rc::StaticRcRef<'a, i32, 2, 2>;
+///
+/// let mut value = 42;
+/// let rc = Two::new(&mut value);
+///
+/// Two::split::<0, 2>(rc);
+/// ```
+pub fn rcref_split_zero_first() {}
+
+/// ```compile_fail,E0080
+/// type Two<'a> = static_rc::StaticRcRef<'a, i32, 2, 2>;
+///
+/// let mut value = 42;
+/// let rc = Two::new(&mut value);
+///
+/// Two::split::<2, 0>(rc);
+/// ```
+pub fn rcref_split_zero_second() {}
+
+/// ```compile_fail,E0080
+/// type Two<'a> = static_rc::StaticRcRef<'a, i32, 2, 2>;
+///
+/// let mut value = 42;
+/// let rc = Two::new(&mut value);
+///
+/// Two::split::<1, 2>(rc);
+/// ```
+pub fn rcref_split_sum() {}
+
+/// ```compile_fail,E0080
+/// type Two<'a> = static_rc::StaticRcRef<'a, i32, 2, 2>;
+///
+/// let mut value = 42;
+/// let rc = Two::new(&mut value);
+///
+/// Two::split_array::<2, 2>(rc);
+/// ```
+pub fn rcref_split_array_ratio() {}
+
+//  FIXME: should be "compile_fail,E0080"
+/// ```should_panic
+/// type Two<'a> = static_rc::StaticRcRef<'a, i32, 2, 2>;
+///
+/// let mut value = 42;
+/// let rc = Two::new(&mut value);
+/// let (one, two) = Two::split::<1, 1>(rc);
+///
+/// static_rc::StaticRcRef::<'_, _, 1, 2>::join(one, two);
+/// ```
+pub fn rcref_join_ratio() {}
+
+//  FIXME: should be "compile_fail,E0080"
+/// ```should_panic
+/// type Two<'a> = static_rc::StaticRcRef<'a, i32, 2, 2>;
+///
+/// let mut value = 42;
+/// let rc = Two::new(&mut value);
+/// let (one, two) = Two::split::<1, 1>(rc);
+///
+/// unsafe { static_rc::StaticRcRef::<'_, _, 1, 2>::join_unchecked(one, two) };
+/// ```
+pub fn rcref_join_unchecked_ratio() {}
+
+//  FIXME: should be "compile_fail,E0080"
+/// ```should_panic
+/// type Two<'a> = static_rc::StaticRcRef<'a, i32, 2, 2>;
+///
+/// let mut value = 42;
+/// let rc = Two::new(&mut value);
+/// let array: [_; 2] = Two::split_array::<1, 2>(rc);
+///
+/// static_rc::StaticRcRef::<'_, _, 1, 2>::join_array(array);
+/// ```
+pub fn rcref_join_array_ratio() {}
+
+//  FIXME: should be "compile_fail,E0080"
+/// ```should_panic
+/// type Two<'a> = static_rc::StaticRcRef<'a, i32, 2, 2>;
+///
+/// let mut value = 42;
+/// let rc = Two::new(&mut value);
+/// let array: [_; 2] = Two::split_array::<1, 2>(rc);
+///
+/// unsafe { static_rc::StaticRcRef::<'_, _, 1, 2>::join_array_unchecked(array) };
+/// ```
+pub fn rcref_join_array_unchecked_ratio() {}
+
+} // mod compile_ratio_tests
+
+#[cfg(all(test, not(feature = "compile-time-ratio")))]
+mod panic_ratio_tests {
+
+use super::*;
+
+type Zero<'a> = StaticRcRef<'a, i32, 0, 0>;
+type One<'a> = StaticRcRef<'a, i32, 1, 1>;
+type Two<'a> = StaticRcRef<'a, i32, 2, 2>;
+
+#[test]
+#[should_panic]
+fn rcref_new_zero() {
+    let mut value = 42;
+
+    Zero::new(&mut value);
 }
 
-} // mod tests
+#[test]
+#[should_panic]
+fn rcref_from_raw_zero() {
+    let pointer = NonNull::dangling();
+
+    unsafe { Zero::from_raw(pointer) };
+}
+
+#[test]
+#[should_panic]
+fn rcref_adjust_zero() {
+    let mut value = 42;
+    let rc = One::new(&mut value);
+
+    One::adjust::<0, 0>(rc);
+}
+
+#[test]
+#[should_panic]
+fn rcref_adjust_ratio() {
+    let mut value = 42;
+    let rc = One::new(&mut value);
+
+    One::adjust::<2, 3>(rc);
+}
+
+#[test]
+#[should_panic]
+fn rcref_split_zero_first() {
+    let mut value = 42;
+    let rc = Two::new(&mut value);
+
+    Two::split::<0, 2>(rc);
+}
+
+#[test]
+#[should_panic]
+fn rcref_split_zero_second() {
+    let mut value = 42;
+    let rc = Two::new(&mut value);
+
+    Two::split::<0, 2>(rc);
+}
+
+#[test]
+#[should_panic]
+fn rcref_split_sum() {
+    let mut value = 42;
+    let rc = Two::new(&mut value);
+
+    Two::split::<1, 2>(rc);
+}
+
+#[test]
+#[should_panic]
+fn rcref_split_array_ratio() {
+    let mut value = 42;
+    let rc = Two::new(&mut value);
+
+    Two::split_array::<2, 2>(rc);
+}
+
+#[test]
+#[should_panic]
+fn rcref_join_ratio() {
+    let mut value = 42;
+    let rc = Two::new(&mut value);
+    let (one, two) = Two::split::<1, 1>(rc);
+
+    StaticRcRef::<'_, _, 1, 2>::join(one, two);
+}
+
+#[test]
+#[should_panic]
+fn rcref_join_different() {
+    let (mut value, mut other_value) = (42, 33);
+    let (rc, other) = (Two::new(&mut value), Two::new(&mut other_value));
+    let (one, _two) = Two::split::<1, 1>(rc);
+    let (other_one, _other_two) = Two::split::<1, 1>(other);
+
+    Two::join(one, other_one);
+}
+
+#[test]
+#[should_panic]
+fn rcref_join_unchecked_ratio() {
+    let mut value = 42;
+    let rc = Two::new(&mut value);
+    let (one, two) = Two::split::<1, 1>(rc);
+
+    unsafe { StaticRcRef::<'_, _, 1, 2>::join_unchecked(one, two) };
+}
+
+#[test]
+#[should_panic]
+fn rcref_join_array_ratio() {
+    let mut value = 42;
+    let rc = Two::new(&mut value);
+    let array: [_; 2] = Two::split_array::<1, 2>(rc);
+
+    StaticRcRef::<'_, _, 1, 2>::join_array(array);
+}
+
+#[test]
+#[should_panic]
+fn rcref_join_array_different() {
+    let (mut value, mut other_value) = (42, 33);
+    let (rc, other) = (Two::new(&mut value), Two::new(&mut other_value));
+    let (one, _two) = Two::split::<1, 1>(rc);
+    let (other_one, _other_two) = Two::split::<1, 1>(other);
+
+    Two::join_array([one, other_one]);
+}
+
+#[test]
+#[should_panic]
+fn rcref_join_array_unchecked_ratio() {
+    let mut value = 42;
+    let rc = Two::new(&mut value);
+    let array = Two::split_array::<1, 2>(rc);
+
+    unsafe { StaticRcRef::<'_, _, 1, 2>::join_array_unchecked(array) };
+}
+
+} // mod panic_ratio_tests
